import logging
from abc import ABC, abstractmethod
from typing import List

from innova_controls.constants import (
    CMD_POWER_OFF,
    CMD_POWER_ON,
    MAX_TEMP,
    MIN_TEMP,
    UNKNOWN_MODE,
)
from innova_controls.fan_speed import FanSpeed
from innova_controls.mode import Mode
from innova_controls.network_functions import NetWorkFunctions

_LOGGER = logging.getLogger(__name__)


class InnovaDevice(ABC):
    class Modes(ABC):
        codes: dict = None

        @classmethod
        def get_supported_modes(cls) -> List[Mode]:
            return cls.codes.values()

        @classmethod
        def get_mode(cls, code: int) -> Mode:
            if code in cls.codes:
                return cls.codes[code]
            else:
                return UNKNOWN_MODE

    def __init__(self, network_facade: NetWorkFunctions) -> None:
        super().__init__()
        self._network_facade = network_facade
        self._data = {}
        self._status = {}

    def set_data(self, data: dict) -> None:
        self._data = data
        if self._data["success"] and "RESULT" in self._data:
            # We don't need the password, so obfuscate it to avoid exposing it in logs
            self._data["RESULT"]["pwd"] = "__OBFUSCATED__"
            _LOGGER.debug(f"Received: {self._data}")
            self._status = self._data["RESULT"]
        else:
            _LOGGER.error("Error contacting the unit with response")

    @property
    @abstractmethod
    def ambient_temp(self) -> float:
        pass

    @property
    @abstractmethod
    def target_temperature(self) -> float:
        pass

    @property
    @abstractmethod
    def fan_speed(self) -> FanSpeed:
        pass

    @property
    @abstractmethod
    def supported_fan_speeds(self) -> List[FanSpeed]:
        pass

    @property
    @abstractmethod
    def rotation(self) -> bool:
        pass

    @property
    @abstractmethod
    def night_mode(self) -> bool:
        pass

    @property
    @abstractmethod
    def temperature_step(self) -> float:
        pass

    @abstractmethod
    async def set_temperature(self, temperature: int) -> bool:
        pass

    @abstractmethod
    async def set_fan_speed(self, speed: FanSpeed) -> bool:
        pass

    @abstractmethod
    async def rotation_on(self) -> bool:
        pass

    @abstractmethod
    async def rotation_off(self) -> bool:
        pass

    @abstractmethod
    async def night_mode_on(self) -> bool:
        pass

    @abstractmethod
    async def night_mode_off(self) -> bool:
        pass

    @abstractmethod
    async def set_heating(self) -> bool:
        pass

    @abstractmethod
    async def set_cooling(self) -> bool:
        pass

    @abstractmethod
    async def set_dehumidifying(self) -> bool:
        pass

    @abstractmethod
    async def set_fan_only(self) -> bool:
        pass

    @abstractmethod
    async def set_auto(self) -> bool:
        pass

    async def _set_mode(self, mode: Mode) -> bool:
        if await self._network_facade._send_command(mode.command):
            self._status["ps"] = 1
            self._status["wm"] = mode.code
            return True
        return False

    @property
    def min_temperature(self) -> int:
        return MIN_TEMP

    @property
    def max_temperature(self) -> int:
        return MAX_TEMP

    @property
    def power(self) -> bool:
        if "ps" in self._status:
            return self._status["ps"] == 1
        return False

    @property
    def mode(self) -> Mode:
        if "wm" in self._status:
            return self.Modes.get_mode(self._status["wm"])
        else:
            return UNKNOWN_MODE

    async def power_on(self) -> bool:
        if await self._network_facade._send_command(CMD_POWER_ON):
            self._status["ps"] = 1
            return True
        return False

    async def power_off(self) -> bool:
        if await self._network_facade._send_command(CMD_POWER_OFF):
            self._status["ps"] = 0
            return True
        return False

    @property
    def name(self) -> str:
        if "setup" in self._data and "name" in self._data["setup"]:
            return self._data["setup"]["name"]
        return None

    @property
    def serial(self) -> str:
        if "setup" in self._data and "serial" in self._data["setup"]:
            return self._data["setup"]["serial"]
        return None

    @property
    def uid(self) -> str:
        if "UID" in self._data:
            return self._data["UID"]
        return None

    @property
    def software_version(self) -> str:
        if "sw" in self._data and "V" in self._data["sw"]:
            return self._data["sw"]["V"]
        return None

    @property
    def ip_address(self) -> str:
        if "net" in self._data and "ip" in self._data["net"]:
            return self._data["net"]["ip"]
        return None

    @property
    def supports_target_temp(self) -> bool:
        return True

    @property
    def supports_swing(self) -> bool:
        return False

    @property
    def supports_fan(self) -> bool:
        return True

    @property
    def supports_preset(self) -> bool:
        return True



from enum import Enum
from typing import List

from innova_controls.constants import CMD_SET_TEMP
from innova_controls.fan_speed import FanSpeed
from innova_controls.innova_device import InnovaDevice
from innova_controls.mode import Mode
from innova_controls.network_functions import NetWorkFunctions


class AirLeaf(InnovaDevice):
    class Modes(InnovaDevice.Modes):
        HEATING = Mode("set/mode/heating", 3, heat=True)
        COOLING = Mode("set/mode/cooling", 5, cool=True)

        codes: dict = {
            3: HEATING,
            5: COOLING,
        }

    class Function(Enum):
        AUTO = {"cmd": "set/function/auto", "code": 1, "fan": FanSpeed.AUTO}
        NIGHT = {"cmd": "set/function/night", "code": 2, "fan": None}
        MIN = {"cmd": "set/function/min", "code": 3, "fan": FanSpeed.LOW}
        MAX = {"cmd": "set/function/max", "code": 4, "fan": FanSpeed.HIGH}

        codes = {1: AUTO, 2: NIGHT, 3: MIN, 4: MAX, "fan": None}

    def __init__(self, network_facade: NetWorkFunctions) -> None:
        super().__init__(network_facade)

    @property
    def temperature_step(self) -> float:
        return 0.5

    @property
    def ambient_temp(self) -> float:
        if "ta" in self._status:
            return self._status["ta"] / 10
        else:
            return 0

    @property
    def target_temperature(self) -> float:
        if "sp" in self._status:
            return self._status["sp"] / 10
        else:
            return 0

    @property
    def water_temperature(self) -> float:
        if "tw" in self._status:
            return self._status["tw"] / 10
        else:
            return 0

    @property
    def fan_speed(self) -> FanSpeed:
        if "fn" in self._status:
            fn = self.Function.codes.value[self._status["fn"]]
            # Night is not really a fan speed, and is handled by preset
            # so, let fan be labeled as AUTO in this case.
            if fn == self.Function.NIGHT:
                fn = FanSpeed.AUTO
            return fn["fan"]
        return FanSpeed.AUTO

    @property
    def supported_fan_speeds(self) -> List[FanSpeed]:
        speeds: List[FanSpeed] = []
        for function in self.Function:
            if "fan" in function.value and function.value["fan"] is not None:
                speeds.append(function.value["fan"])
        return speeds

    @property
    def rotation(self) -> bool:
        return False

    @property
    def night_mode(self) -> bool:
        if "fn" in self._status:
            if self._status["fn"] == self.Function.NIGHT.value["code"]:
                return True
        return False

    async def set_temperature(self, temperature: int) -> bool:
        new_temp = temperature * 100
        data = {"temp": new_temp}
        if await self._network_facade._send_command(CMD_SET_TEMP, data):
            self._status["sp"] = new_temp
            return True
        return False

    async def set_fan_speed(self, speed: FanSpeed) -> bool:
        command: str = None
        code: int = None
        for function in self.Function:
            if speed == function.value["fan"]:
                command = function.value["cmd"]
                code = function.value["code"]
                break

        if command and await self._network_facade._send_command(command):
            self._status["fn"] = code
            return True
        return False

    async def rotation_on(self) -> bool:
        return False

    async def rotation_off(self) -> bool:
        return False

    async def night_mode_on(self) -> bool:
        if await self._network_facade._send_command(self.Function.NIGHT.value["cmd"]):
            self._status["fn"] = self.Function.NIGHT.value["code"]
            return True
        return False

    async def night_mode_off(self) -> bool:
        if await self._network_facade._send_command(self.Function.AUTO.value["cmd"]):
            self._status["fn"] = self.Function.AUTO.value["code"]
            return True
        return False

    async def set_heating(self) -> bool:
        if self.power or await self.power_on():
            return await self._set_mode(self.Modes.HEATING)
        return False

    async def set_cooling(self) -> bool:
        if self.power or await self.power_on():
            return await self._set_mode(self.Modes.COOLING)
        return False

    async def set_auto(self) -> bool:
        pass

    async def set_dehumidifying(self) -> bool:
        pass

    async def set_fan_only(self) -> bool:
        pass



import logging
from typing import List

from aiohttp import ClientSession

from innova_controls.constants import UNKNOWN_MODE
from innova_controls.fan_speed import FanSpeed
from innova_controls.innova_device import InnovaDevice
from innova_controls.innova_factory import InnovaFactory
from innova_controls.mode import Mode
from innova_controls.network_functions import NetWorkFunctions

_LOGGER = logging.getLogger(__name__)


class Innova:
    """This is a class to control Innova heat pump units over http

    Attributes:
        For Local Mode
        host: str
            IP address of the Innova unit on the local network.
            If omitted, cloud mode is assumed

        For Cloud Mode (both serial and uid are mandatory)
        serial: str
            Serial number of the Innova unit (usually looks like INXXXXXXX)
        uid: str)
            The MAC address of the Innova unit.
    """

    def __init__(
        self,
        http_session: ClientSession,
        host: str = None,
        serial: str = None,
        uid: str = None,
    ):
        _LOGGER.info(
            f"Initialize Innova Controls with host={host}, "
            "serial={serial}, uid={uid}"
        )

        self._network_facade = NetWorkFunctions(http_session, host, serial, uid)
        self._innova_device: InnovaDevice = None

    async def async_update(self) -> bool:
        data: dict = await self._network_facade.get_status()

        if data:
            if self._innova_device is None:
                self._innova_device = InnovaFactory.get_device(
                    data["deviceType"], self._network_facade
                )
            self._innova_device.set_data(data)
            _LOGGER.debug(f"Received: {data}")
            return True
        else:
            _LOGGER.error(f"Error retrieving unit status")
            return False

    @property
    def ambient_temp(self) -> float:
        if self._innova_device:
            return self._innova_device.ambient_temp
        return 0

    @property
    def target_temperature(self) -> float:
        if self._innova_device:
            return self._innova_device.target_temperature
        return 0

    @property
    def temperature_step(self) -> float:
        if self._innova_device:
            return self._innova_device.temperature_step
        return 1.0

    @property
    def min_temperature(self) -> float:
        if self._innova_device:
            return self._innova_device.min_temperature
        return 0

    @property
    def max_temperature(self) -> float:
        if self._innova_device:
            return self._innova_device.max_temperature
        return 0

    @property
    def power(self) -> bool:
        if self._innova_device:
            return self._innova_device.power
        return False

    @property
    def mode(self) -> Mode:
        if self._innova_device:
            return self._innova_device.mode
        return UNKNOWN_MODE

    @property
    def supported_modes(self) -> List[Mode]:
        if self._innova_device:
            return list(self._innova_device.Modes.get_supported_modes())
        return []

    @property
    def rotation(self) -> bool:
        if self._innova_device:
            return self._innova_device.rotation
        return False

    @property
    def fan_speed(self) -> FanSpeed:
        if self._innova_device:
            return self._innova_device.fan_speed
        return 0

    @property
    def supported_fan_speeds(self) -> List[FanSpeed]:
        if self._innova_device:
            return self._innova_device.supported_fan_speeds
        return []

    @property
    def night_mode(self) -> bool:
        if self._innova_device:
            return self._innova_device.night_mode
        return False

    @property
    def name(self) -> str:
        if self._innova_device:
            return self._innova_device.name
        return None

    @property
    def serial(self) -> str:
        if self._innova_device:
            return self._innova_device.serial
        return None

    @property
    def uid(self) -> str:
        if self._innova_device:
            return self._innova_device.uid
        return None

    @property
    def software_version(self) -> str:
        if self._innova_device:
            return self._innova_device.software_version
        return None

    @property
    def ip_address(self) -> str:
        if self._innova_device:
            return self._innova_device.ip_address
        return None

    async def power_on(self) -> bool:
        if self._innova_device:
            return await self._innova_device.power_on()
        return False

    async def power_off(self) -> bool:
        if self._innova_device:
            return await self._innova_device.power_off()
        return False

    async def rotation_on(self) -> bool:
        if self._innova_device:
            return await self._innova_device.rotation_on()
        return False

    async def rotation_off(self) -> bool:
        if self._innova_device:
            return await self._innova_device.rotation_off()
        return False

    async def night_mode_on(self) -> bool:
        if self._innova_device:
            return await self._innova_device.night_mode_on()
        return False

    async def night_mode_off(self) -> bool:
        if self._innova_device:
            return await self._innova_device.night_mode_off()
        return False

    async def set_temperature(self, temperature: float) -> bool:
        if self._innova_device:
            return await self._innova_device.set_temperature(temperature)
        return False

    async def set_fan_speed(self, speed: FanSpeed) -> bool:
        if self._innova_device:
            return await self._innova_device.set_fan_speed(speed)
        return False

    async def set_heating(self) -> bool:
        return await self._innova_device.set_heating()

    async def set_cooling(self) -> bool:
        return await self._innova_device.set_cooling()

    async def set_dehumidifying(self) -> bool:
        return await self._innova_device.set_dehumidifying()

    async def set_fan_only(self) -> bool:
        return await self._innova_device.set_fan_only()

    async def set_auto(self) -> bool:
        return await self._innova_device.set_auto()

    @property
    def supports_target_temp(self) -> bool:
        return self._innova_device.supports_target_temp

    @property
    def supports_swing(self) -> bool:
        return self._innova_device.supports_swing

    @property
    def supports_fan(self) -> bool:
        return self._innova_device.supports_fan

    @property
    def supports_preset(self) -> bool:
        return self._innova_device.supports_preset



import logging

from aiohttp import ClientConnectionError, ClientSession, ServerTimeoutError
from retry import retry

from innova_controls.constants import CMD_STATUS, CONNECTION_TIMEOUT

_LOGGER = logging.getLogger(__name__)


class NetWorkFunctions:
    def __init__(
        self,
        http_session: ClientSession,
        host: str = None,
        serial: str = None,
        uid: str = None,
    ) -> None:

        self._http_session = http_session

        if host is not None:
            # Setup for local mode
            _LOGGER.debug("Setting up local mode")
            self._api_url = f"http://{host}/api/v/1"
            self._headers = None
        else:
            # Setup for cloud mode
            _LOGGER.debug("Setting up cloud mode")
            self._api_url = "http://innovaenergie.cloud/api/v/1"
            self._headers = {"X-serial": serial, "X-UID": uid}

    @retry(exceptions=Exception, tries=2, delay=2, logger=_LOGGER, log_traceback=True)
    async def _send_command(self, command, data=None) -> bool:
        cmd_url = f"{self._api_url}/{command}"
        try:
            r = await self._http_session.post(
                cmd_url, data=data, headers=self._headers, timeout=CONNECTION_TIMEOUT
            )

            if r.status == 200:
                result = await r.json(content_type=r.content_type)
                if result["success"]:
                    return True
            return False
        except ServerTimeoutError:
            return False
        except ClientConnectionError:
            return False
        except Exception as e:
            _LOGGER.error(f"Error while sending command {cmd_url}", e)
            return False

    @retry(exceptions=Exception, tries=2, delay=2, logger=_LOGGER, log_traceback=True)
    async def get_status(self) -> dict:
        status_url = f"{self._api_url}/{CMD_STATUS}"
        try:
            r = await self._http_session.get(
                status_url, headers=self._headers, timeout=CONNECTION_TIMEOUT
            )
            data: dict = await r.json(content_type=r.content_type)
            if data and data["success"] and "RESULT" in data:
                return data
            else:
                _LOGGER.error(f"Error contacting the unit with response {r.text}")
                return None
        except Exception as e:
            _LOGGER.error(f"Error getting status {status_url}", e)
            return None



from enum import Enum

from innova_controls.airleaf import AirLeaf
from innova_controls.innova_device import InnovaDevice
from innova_controls.network_functions import NetWorkFunctions
from innova_controls.twopointzero import TwoPointZero


class DeviceType(Enum):
    TWOPOINTZERO = "001"
    AIRLEAF = "002"


class InnovaFactory:
    @staticmethod
    def get_device(device_type: str, network_facade: NetWorkFunctions) -> InnovaDevice:
        if device_type == DeviceType.TWOPOINTZERO.value:
            return TwoPointZero(network_facade)
        elif device_type == DeviceType.AIRLEAF.value:
            return AirLeaf(network_facade)



class Mode:
    def __init__(
        self,
        command: str,
        code: int,
        heat: bool = False,
        cool: bool = False,
        dehumidify: bool = False,
        fan_only: bool = False,
        auto: bool = False,
    ) -> None:
        self.command = command
        self.code = code

        self._heat = heat
        self._cool = cool
        self._dehumidify = dehumidify
        self._fan_only = fan_only
        self._auto = auto

    @property
    def is_heating(self) -> bool:
        return self._heat

    @property
    def is_cooling(self) -> bool:
        return self._cool

    @property
    def is_dehumidifying(self) -> bool:
        return self._dehumidify

    @property
    def is_fan_only(self) -> bool:
        return self._fan_only

    @property
    def is_auto(self) -> bool:
        return self._auto

    def __repr__(self) -> str:
        return (
            f"Mode(Code: {self.code}, Command: {self.command}, "
            f"Heat: {self._heat}, Cool: {self._cool}, Dehumidify: {self._dehumidify}, "
            f"Fan Only: {self._fan_only}, Auto: {self._auto})"
        )



from innova_controls.mode import Mode

CMD_POWER_ON = "power/on"
CMD_POWER_OFF = "power/off"
CMD_NIGHT_MODE = "set/feature/night"
CMD_SET_TEMP = "set/setpoint"
CMD_ROTATION = "set/feature/rotation"
CMD_FAN_SPEED = "set/fan"
CMD_STATUS = "status"

ROTATION_ON = 0
ROTATION_OFF = 7

NIGHT_MODE_ON = 1
NIGHT_MODE_OFF = 0

MIN_TEMP = 16
MAX_TEMP = 31

CONNECTION_TIMEOUT = 20

UNKNOWN_MODE = Mode("", -1)



from enum import Enum
from typing import List

from innova_controls.constants import (
    CMD_FAN_SPEED,
    CMD_NIGHT_MODE,
    CMD_ROTATION,
    CMD_SET_TEMP,
    NIGHT_MODE_OFF,
    NIGHT_MODE_ON,
    ROTATION_OFF,
    ROTATION_ON,
)
from innova_controls.fan_speed import FanSpeed
from innova_controls.innova_device import InnovaDevice
from innova_controls.mode import Mode
from innova_controls.network_functions import NetWorkFunctions


class TwoPointZero(InnovaDevice):
    class Modes(InnovaDevice.Modes):
        HEATING = Mode("set/mode/heating", 0, heat=True)
        COOLING = Mode("set/mode/cooling", 1, cool=True)
        DEHUMIDIFICATION = Mode("set/mode/dehumidification", 3, dehumidify=True)
        FAN_ONLY = Mode("set/mode/fanonly", 4, fan_only=True)
        AUTO = Mode("set/mode/auto", 5, auto=True)

        codes: dict = {
            0: HEATING,
            1: COOLING,
            3: DEHUMIDIFICATION,
            4: FAN_ONLY,
            5: AUTO,
        }

    fan_speeds = {
        0: FanSpeed.AUTO,
        1: FanSpeed.LOW,
        2: FanSpeed.MEDIUM,
        3: FanSpeed.HIGH,
    }
    fan_speeds_reverse: dict = {v: k for k, v in fan_speeds.items()}

    def __init__(self, network_facade: NetWorkFunctions) -> None:
        super().__init__(network_facade)

    @property
    def temperature_step(self) -> float:
        return 1.0

    @property
    def ambient_temp(self) -> float:
        if "t" in self._status:
            return self._status["t"]
        else:
            return 0

    @property
    def target_temperature(self) -> float:
        if "sp" in self._status:
            return self._status["sp"]
        else:
            return 0

    @property
    def fan_speed(self) -> FanSpeed:
        if "fs" in self._status:
            return self.fan_speeds[self._status["fs"]]
        return FanSpeed.AUTO

    @property
    def supported_fan_speeds(self) -> List[FanSpeed]:
        return list(self.fan_speeds.values())

    @property
    def rotation(self) -> bool:
        if "fr" in self._status:
            if self._status["fr"] == ROTATION_ON:
                return True
        return False

    @property
    def night_mode(self) -> bool:
        if "nm" in self._status:
            if self._status["nm"] == NIGHT_MODE_ON:
                return True
        return False

    async def set_temperature(self, temperature: int) -> bool:
        data = {"p_temp": temperature}
        if await self._network_facade._send_command(CMD_SET_TEMP, data):
            self._status["sp"] = temperature
            return True
        return False

    async def set_fan_speed(self, speed: FanSpeed) -> bool:
        speed_code = self.fan_speeds_reverse[speed]
        data = {"value": speed_code}
        if await self._network_facade._send_command(CMD_FAN_SPEED, data):
            self._status["fs"] = speed_code
            return True
        return False

    @property
    def supports_swing(self) -> bool:
        return True

    async def rotation_on(self) -> bool:
        if await self._network_facade._send_command(
            CMD_ROTATION, {"value": ROTATION_ON}
        ):
            self._status["fr"] = ROTATION_ON
            return True
        return False

    async def rotation_off(self) -> bool:
        if await self._network_facade._send_command(
            CMD_ROTATION, {"value": ROTATION_OFF}
        ):
            self._status["fr"] = ROTATION_OFF
            return True
        return False

    async def night_mode_on(self) -> bool:
        if await self._network_facade._send_command(
            CMD_NIGHT_MODE, {"value": NIGHT_MODE_ON}
        ):
            self._status["nm"] = NIGHT_MODE_ON
            return True
        return False

    async def night_mode_off(self) -> bool:
        if await self._network_facade._send_command(
            CMD_NIGHT_MODE, {"value": NIGHT_MODE_OFF}
        ):
            self._status["nm"] = NIGHT_MODE_OFF
            return True
        return False

    async def set_heating(self) -> bool:
        return await self._set_mode(self.Modes.HEATING)

    async def set_cooling(self) -> bool:
        return await self._set_mode(self.Modes.COOLING)

    async def set_dehumidifying(self) -> bool:
        return await self._set_mode(self.Modes.DEHUMIDIFICATION)

    async def set_fan_only(self) -> bool:
        return await self._set_mode(self.Modes.FAN_ONLY)

    async def set_auto(self) -> bool:
        return await self._set_mode(self.Modes.AUTO)



"""A setuptools based setup module.

See:
https://packaging.python.org/guides/distributing-packages-using-setuptools/
https://github.com/pypa/sampleproject
"""

from setuptools import setup
import pathlib

here = pathlib.Path(__file__).parent.resolve()

# Get the long description from the README file
long_description = (here / "README.md").read_text(encoding="utf-8")

setup(
    name="innova-controls",
    version="2.1.3",
    description="Innova Air Conditioner Control API",
    license="Apache",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/danielrivard/innova-controls",
    author="Daniel Rivard",
    # author_email='author@example.com',
    keywords="development, home automation, library, innova",
    packages=["innova_controls"],
    # https://packaging.python.org/guides/distributing-packages-using-setuptools/#python-requires
    python_requires=">=3.9, <4",
    install_requires=["aiohttp >= 3.0.0, < 4.0.0", "retry2>=0.9.3"],
    project_urls={
        "Bug Reports": "https://github.com/danielrivard/innova-controls/issues",
        "Source": "https://github.com/danielrivard/innova-controls/",
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Software Development :: Libraries",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
    ],
)
